package com.shri.publisher;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ExecutionException;

import javax.annotation.PostConstruct;

import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.common.serialization.StringSerializer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.kafka.core.DefaultKafkaProducerFactory;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.support.SendResult;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.concurrent.ListenableFuture;

/**
 * This publisher class directly uses the spring kafka for publishing
 * @author spatel
 *
 */

@Component
@EnableAsync
@EnableScheduling
public class SpringKafkaPublisher {

	public static Logger logger = LoggerFactory.getLogger(SpringKafkaPublisher.class);
	private static final String TEMPLATE_TOPIC = "my-topic";
	private static final String BOOTSTRAP_SERVERS_CONFIG = "localhost:9092";//"cd-arch-006:9092";
	private static int eventId = 0;
	private KafkaTemplate<String, String> kafkaTemplate;

	/**
	 * Init the kafka producer factory and kafka template
	 * 
	 * @throws Exception
	 */
	@PostConstruct
	public void init() throws Exception {
		Map<String, Object> producerProps = new HashMap<String, Object>();
		producerProps.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, StringSerializer.class);
		producerProps.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, StringSerializer.class);
		producerProps.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, BOOTSTRAP_SERVERS_CONFIG);

		DefaultKafkaProducerFactory<String, String> producerFactory = new DefaultKafkaProducerFactory<String, String>(
				producerProps);
		kafkaTemplate = new KafkaTemplate<String, String>(producerFactory, true);

	}

	/**
	 * publish message to kafka
	 * @throws InterruptedException 
	 */
	@Scheduled(cron = "${cron.pattern}")
	public void publishMessage() throws InterruptedException {

		int i = eventId++;
		String key = Integer.toString(i);
		String value = "value " + i;

		ListenableFuture<SendResult<String, String>> future = kafkaTemplate.send(TEMPLATE_TOPIC, 0, key, value);
		try {
			SendResult<String, String> res = future.get();
		} catch (ExecutionException e) {
			e.printStackTrace();
		}
	}

}