package com.shri.subscriber;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.kafka.core.DefaultKafkaConsumerFactory;
import org.springframework.kafka.listener.AbstractMessageListenerContainer.AckMode;
import org.springframework.kafka.listener.AcknowledgingMessageListener;
import org.springframework.kafka.listener.ConcurrentMessageListenerContainer;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.stereotype.Component;

/**
 * This Subscriber class directly uses spring kafka for subscribing
 * 
 * @author spatel
 *
 */

@Component("SpringKafkaSubscriber")
public class SpringKafkaSubscriber {

	public static Logger logger = LoggerFactory.getLogger(SpringKafkaSubscriber.class);
	private static final String TEMPLATE_TOPIC = "my-topic";// "my-topic";//
	private static final int listener_concurrency = 1;
	private static final String BOOTSTRAP_SERVERS_CONFIG = "localhost:9092";// "cd-arch-006:9092";
	private static final String CONSUMER_GROUP = "group0";

	public static void start() throws IOException {

		Map<String, Object> consumerProps = new HashMap<String, Object>();
		consumerProps.put(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class);
		consumerProps.put(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class);
		consumerProps.put(ConsumerConfig.ENABLE_AUTO_COMMIT_CONFIG, Boolean.FALSE);
		consumerProps.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, BOOTSTRAP_SERVERS_CONFIG);
		consumerProps.put(ConsumerConfig.GROUP_ID_CONFIG, CONSUMER_GROUP);
		consumerProps.put(ConsumerConfig.AUTO_OFFSET_RESET_CONFIG, "earliest");

		DefaultKafkaConsumerFactory<String, String> cf = new DefaultKafkaConsumerFactory<String, String>(consumerProps);
		ConcurrentMessageListenerContainer<String, String> container = new ConcurrentMessageListenerContainer<String, String>(
				cf, TEMPLATE_TOPIC);

		container.setAckMode(AckMode.MANUAL_IMMEDIATE);
		container.setConcurrency(listener_concurrency);

		container.setMessageListener(new AcknowledgingMessageListener<String, String>() {

			@Override
			public void onMessage(ConsumerRecord<String, String> message, Acknowledgment ack) {
				logger.info("received: key - " + message.key() + " value - " + message.value());
				ack.acknowledge();
			}

		});
		container.start();

	}
}
