
import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.Assert.assertEquals;

import java.util.Map;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;

import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.junit.ClassRule;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.kafka.core.DefaultKafkaConsumerFactory;
import org.springframework.kafka.core.DefaultKafkaProducerFactory;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.listener.AbstractMessageListenerContainer.AckMode;
import org.springframework.kafka.listener.AcknowledgingMessageListener;
import org.springframework.kafka.listener.ConcurrentMessageListenerContainer;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.kafka.test.rule.KafkaEmbedded;
import org.springframework.kafka.test.utils.KafkaTestUtils;

/**
 * @author spatel
 */
//@RunWith(SpringJUnit4ClassRunner.class)
//@ContextConfiguration(loader = AnnotationConfigContextLoader.class)
public class SpringKakfaSubscriberTest {

	private static final String TEMPLATE_TOPIC = "my-topic";

	public static Logger logger = LoggerFactory.getLogger(SpringKakfaSubscriberTest.class);

	@ClassRule
	public static KafkaEmbedded embeddedKafka = new KafkaEmbedded(1, true, 1, TEMPLATE_TOPIC);
	private int CONSUMER_COUNT;

	@Test
	public void consumeEvent() throws Exception {

		Map<String, Object> producerProps = KafkaTestUtils.producerProps(embeddedKafka);

		DefaultKafkaProducerFactory<Integer, String> pf = new DefaultKafkaProducerFactory<Integer, String>(
				producerProps);
		KafkaTemplate<Integer, String> kafkaTemplate = new KafkaTemplate<Integer, String>(pf);
		kafkaTemplate.setDefaultTopic(TEMPLATE_TOPIC);

		int PRODUCER_COUNT = 5;
		CONSUMER_COUNT = 0;

		for (int i = 0; i < PRODUCER_COUNT; i++) {
			Integer key = i;
			String value = "value " + i;
			kafkaTemplate.send(key, value);
		}
		kafkaTemplate.flush();

		Map<String, Object> consumerProps = KafkaTestUtils.consumerProps("group", "false", embeddedKafka);
		consumerProps.put(ConsumerConfig.AUTO_OFFSET_RESET_CONFIG, "earliest");
		DefaultKafkaConsumerFactory<Integer, String> cf = new DefaultKafkaConsumerFactory<Integer, String>(consumerProps);
		ConcurrentMessageListenerContainer<Integer, String> container = new ConcurrentMessageListenerContainer<Integer, String>(
				cf, TEMPLATE_TOPIC);
		container.setAckMode(AckMode.MANUAL_IMMEDIATE);
		container.setConcurrency(1);
		final CountDownLatch latch = new CountDownLatch(PRODUCER_COUNT);

		container.setMessageListener(new AcknowledgingMessageListener<Integer, String>() {
			@Override
			public void onMessage(ConsumerRecord<Integer, String> message, Acknowledgment ack) {
				logger.error("********************* received: key - " + message.key() + " value - " + message.value());
				ack.acknowledge();
				incrementCounter();
				latch.countDown();
			}
		});
		
		container.start();
		assertThat(latch.await(15, TimeUnit.SECONDS)).isTrue();
		container.stop();
		assertEquals(PRODUCER_COUNT, CONSUMER_COUNT);
		CONSUMER_COUNT = 0;
		container.start();
		Thread.sleep(15000);
		container.stop();
		logger.error("********************* count - " + CONSUMER_COUNT);
		assertEquals(0, CONSUMER_COUNT); 
		
	}

	public final void incrementCounter() {
		CONSUMER_COUNT++;
	}

}